import grafica.*;
import processing.serial.*;
// Need G4P library
import g4p_controls.*;

Serial serial;

String portName;
int portSpeed = 115200;
boolean serialFlag = false;
int moveFlag = 0;
int moveSpeed;
public String voltage, current, power = "";
int tmr1;
final int MY_PERIOD = 500;

GPlot plotU;
GPlot plotI;
GPointsArray pointsU;
GPointsArray pointsI;

public void setup(){
  size(1280, 800, JAVA2D);
  createGUI();
  customGUI();
  plotU = new GPlot(this, 10, 310);
  plotI = new GPlot(this, 475, 310);
  plotU.setTitleText("Напряжение");
  plotU.getXAxis().setAxisLabelText("Время, с");
  plotU.getYAxis().setAxisLabelText("Напряжение, В");
  plotI.setTitleText("Ток");
  plotI.getXAxis().setAxisLabelText("Время, с");
  plotI.getYAxis().setAxisLabelText("Ток, А");
  plotU.activatePanning();
  plotU.activateZooming(1.1, CENTER, CENTER);
  plotI.activatePanning();
  plotI.activateZooming(1.1, CENTER, CENTER);
  pointsU = new GPointsArray();
  pointsI = new GPointsArray();
}

public void draw(){
  background(230);
  if(serial != null){
    if(serial.available() > 0){
      String str = serial.readStringUntil(';');
      if(str != null) {
        String data[] = str.split("/");
        //println(str);
        if(data.length == 3) {
          voltage = data[0];
          current = data[1];
          power = data[2];
        }
      }
    }
    if(serialFlag){
      serial.write("srv/" + slider_servo.getValueI() + "\n");
      if (moveFlag == 1) {
        int pos = joy.getPosition();
        if ((pos == 0) || ((keyPressed == true) && (key == 'd'))) {
          serial.write("move/trn_r/" + moveSpeed + "\n");
        } else if ((pos == 2) || ((keyPressed == true) && (key == 's'))) {
          serial.write("move/bck/" + moveSpeed + "\n");
        } else if ((pos == 4) || ((keyPressed == true) && (key == 'a'))) {
          serial.write("move/trn_l/" + moveSpeed + "\n");
        } else if ((pos == 6) || ((keyPressed == true) && (key == 'w'))) {
          serial.write("move/frw/" + moveSpeed + "\n");
        } else if ((pos == -1) && (keyPressed == false)) {
          serial.write("move/stop\n");
        }
      }
      if(plat_power_on.isSelected()){
        serial.write("ptpw/1\n");
      } else if(plat_power_off.isSelected()){
        serial.write("ptpw/0\n");
      }
    }
  }
  voltage_label.setText("");
  voltage_label.setText("Напряжение: " + voltage + " В");
  current_label.setText("");
  current_label.setText("Ток: " + current + " А");
  power_label.setText("");
  power_label.setText("Мощность: " + power + " Вт");
  if((voltage != null) && (current != null) && (power != null) && (serialFlag == true)) {
    if (millis() - tmr1 >= MY_PERIOD) { 
      tmr1 = millis();                   
      pointsU.add(millis()/1000, float(voltage));
      pointsI.add(millis()/1000, float(current));
      plotU.setPoints(pointsU);
      plotI.setPoints(pointsI);
    }
  }
  plotU.defaultDraw();
  plotI.defaultDraw();
  println(keyPressed);
}

/*
public void kmlSelected(File selection){
  kml = loadXML(selection.getAbsolutePath());
  XML firstChild = kml.getChild("kml");
  println(firstChild.getContent());
}
*/

// Use this method to add additional statements
// to customise the GUI controls
public void customGUI(){

}
